;-----------------------------------------------------------------------
; AutoHotkey script to create an animated GIF from photobooth photos.
;
; Usage: Double click on this script to install it (this updates the PhotoboothStatusCmdXML
; setting in the Windows registry). Double click on the script again if you wish to uninstall it.
;
; iIMPORTANT: Select the option to create PNG copies of the GIF frames in the animated GIF settings
; The script creates the MP4 file from the saved frames as this will give full color. If it created
; the MP4 file from the animated GIF it would be limited to 256 colors per frame.
;
; Once installed run the photobooth software as normal and the script will be run automatically
; when it saves the XML summary file.
;
; Edit the outputDir to specify the subfolder where the MP4 file should be saved.
; Edit minMP4duration to specify the minimum duration of the MP4 file. This is useful if the
; MP4 is to be uploaded to Instagram which requires a minimum duration of 3 secs for videos.
; If the duration is less than the minimum duration the frames are repeated as many times as
; is necessary to make the MP4 long enough.
;
; AutoHotKey is free and can be downloaded from:
; http://www.autohotkey.com
;
; This script comes with no warranty or support whatsoever and may
; be freely copied or modified as required.
;
; Written by Chris Breeze, www.breezesys.com
; 20160913: Inital version
;-----------------------------------------------------------------------
#SingleInstance,force
#NoEnv
SetWorkingDir %A_ScriptDir%

; folder where MP4 is saved (Set this to prints to save the MP4 in the same folder
; as the JPEG copies of the printed output from the photo booth)
outputDir = MP4

; set this to the minimum duration (in 1/100 sec) for the MP4
; e.g. set it to 300 to create a 3 sec video suitable for posting on Instagram
minMP4duration := 300

;*******************************************************************
;************ No need to change anything below here ****************
;*******************************************************************

ifexist,%A_ScriptDir%\ffmpeg.exe
{
	ffmpegExe = %A_ScriptDir%\ffmpeg.exe
}
else
{
	MsgBox, ffmpeg not found, please make sure that ffmpeg.exe is in the same folder as this script
	exitapp
}

IfNotExist, %1%
{
	; no file passed via command line: install or uninstall the script
	Install()
	exitapp
}

; Extract filename and directory from XML pathname
dir = %1%
RegExMatch(dir, "([^\\]+)$", filename) 
RegExMatch(dir, "(.*)\\", dir) 

SetWorkingDir %dir%
FileCreateDir %outputDir%

; read info from XML file
FileRead, Contents, %1%
RegExMatch(Contents, "<image_delay>(.*)</image_delay>", image_delay) 
RegExMatch(Contents, "<title_delay>(.*)</title_delay>", title_delay) 
RegExMatch(Contents, "<title_at_end>(.*)</title_at_end>", title_at_end) 
RegExMatch(Contents, "<gif_frames_subfolder>(.*)</gif_frames_subfolder>", gif_frames_subfolder) 
RegExMatch(Contents, "<gif_file>(.*)</gif_file>", gif_file) 
RegExMatch(Contents, "<gif_file>.*\\(.*)\..*</gif_file>", prefix) 

SetWorkingDir %gif_frames_subfolder1%

imageDelay := 75
imageDelayDenominator := 75
imageDelayNumerator := 100
if image_delay1 > 0
{
	imageDelay := image_delay1
	imageDelayDenominator := image_delay1
	if gif_overlays_per_photo1 > 0
	{
		imageDelayNumerator := 100 * gif_overlays_per_photo1
	}
}
titleDelay := 150
if title_delay1 > 0
{
	titleDelay := title_delay1
}

mp4 := prefix1 . ".mp4"

; list the frames
numFrames := 0
frames :=
loop %prefix1%_frame*.png
{
	numFrames++
	if frames
	{
		frames := frames . "`n" . A_loopFilename
	}
	else
	{
		frames := A_loopFilename
	}
}
if numFrames < 1
{
	exitapp
}
sort,frames
frames := StrReplace(frames, "`n", "|")

; list the title pages
; slight simplification: add title pages multiple times until the title delay exceeds the image delay
; i.e. each title page is displayed n * imageDelay where n * imageDelay >= titleDelay
numTitlePages := 0
titlePages :=
loop %prefix1%_title*.png
{
	duration := 0
	while duration < titleDelay
	{
		duration := duration + imageDelay
		numTitlePages++
		if titlePages
		{
			titlePages := titlePages . "`n" . A_loopFilename
		}
		else
		{
			titlePages := A_loopFilename
		}
	}
}
sort,titlePages
titlePages := StrReplace(titlePages, "`n", "|")

; repeat images until minDuration is reached
minDuration := minMP4duration * imageDelayNumerator
framesCopy := frames
numFramesCopy := numFrames
loop
{
	duration := (numTitlePages + numFrames) * imageDelayDenominator * 100
	if duration >= %minDuration%
	{
		break
	}
	frames := frames . "|" . framesCopy
	numFrames := numFrames + numFramesCopy
}

; add title pages (at front or back)
if numTitlePages > 0
{
	if title_at_end1 > 0
	{
		; titles at end
		frames := frames . "|" . titlePages
	}
	else
	{
		; titles at start
		frames := titlePages . "|" . frames
	}
	numFrames := numFrames + numTitlePages
}

cmd = %ffmpegExe% -framerate %imageDelayNumerator%/%imageDelayDenominator% -i concat:%frames% -vf "scale=trunc(iw/2)*2:trunc(ih/2)*2" -r 25 -c:v libx264 -preset veryslow -pix_fmt yuv420p %mp4%
;MsgBox image_delay: %image_delay1%`ntitle_delay: %title_delay1%`nnumFrames: %numFrames%`nnumTitlePages: %numTitlePages%`n%cmd%

FileDelete,%mp4%
runwait %cmd%,,hide

SetWorkingDir %dir%
FileMove,%gif_frames_subfolder1%\%mp4%,%outputDir%,1

exitapp

; display a window asking user whether to install or uninstall the script for each of
; the installed photobooth programs
Install()
{
	global DSLRRemoteState, NKRemoteState, WebcamState, PSRemoteState
	newCmd = "%A_AhkPath%" "%A_ScriptFullPath%"
	DSLRKey = Software\BreezeSystems\DSLRRemotePro\100
	NKRKey = Software\BreezeSystems\NKRemote\100
	WebKey = Software\BreezeSystems\WebcamPhotobooth\101
	PSRKey = Software\BreezeSystems\PSRemote\102
	Gui, Add, Text,, Install/uninstall photo booth MP4 script
	Gui, Add, Text,, Please select item below to enable this script:
	RegRead, exe, HKEY_CURRENT_USER, %DSLRKey%, exe
	IfExist,%exe%
	{
		RegRead, enabled, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdEnable
		RegRead, cmd, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdXML

		if (cmd = newCmd and enabled = 1)
		{
			Gui, Add, Checkbox, gDSLRRemote checked vDSLRRemoteState, DSLR Remote Pro for Windows
		}
		else
		{
			Gui, Add, Checkbox, gDSLRRemote vDSLRRemoteState, DSLR Remote Pro for Windows
		}
	}
	RegRead, exe, HKEY_CURRENT_USER, %NKRKey%, exe
	IfExist,%exe%
	{
		RegRead, enabled, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdEnable
		RegRead, cmd, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdXML

		if (cmd = newCmd and enabled = 1)
		{
			Gui, Add, Checkbox, gNKRemote checked vNKRemoteState, NKRemote
		}
		else
		{
			Gui, Add, Checkbox, gNKRemote vNKRemoteState, NKRemote
		}
	}
	RegRead, exe, HKEY_CURRENT_USER, Software\BreezeSystems\WebcamPhotobooth\100, exe
	StringReplace,exe,exe,_,,1
	IfExist,%exe%
	{
		RegRead, enabled, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdEnable
		RegRead, cmd, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdXML

		if (cmd = newCmd and enabled = 1)
		{
			Gui, Add, Checkbox, gWebcam checked vWebcamState, Webcam Photobooth
		}
		else
		{
			Gui, Add, Checkbox, gWebcam vWebcamState, Webcam Photobooth
		}
	}
	RegRead, exe, HKEY_CURRENT_USER, Software\BreezeSystems\PSRemote\100, exe
	IfExist,%exe%
	{
		RegRead, enabled, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdEnable
		RegRead, cmd, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdXML

		if (cmd = newCmd and enabled = 1)
		{
			Gui, Add, Checkbox, gPSRemote checked vPSRemoteState, PSRemote
		}
		else
		{
			Gui, Add, Checkbox, gPSRemote vPSRemoteState, PSRemote
		}
	}

	Gui, Add, Button, Default, Close
	Gui, Show
	loop
	{
		Sleep,100
	}
DSLRRemote:
	Gui, Submit,NoHide
	ifWinExist,DSLR Remote Pro for Windows -
	{
		MsgBox,Please exit DSLR Remote Pro before installing/uninstalling this script
		RegRead, enabled, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdEnable
		if enabled
		{
			GuiControl,,DSLRRemoteState,1
		}
		else
		{
			GuiControl,,DSLRRemoteState,0
		}
		return
	}
	if DSLRRemoteState
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdEnable, 1
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothSuppressXml, 0
		RegWrite, REG_SZ, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdXML, %newCmd%
		MsgBox,8192,%A_ScriptName%,Script enabled for DSLR Remote Pro,1000
	}
	else
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdEnable, 0
		MsgBox,8192,%A_ScriptName%,Script disabled for DSLR Remote Pro,1000
	}
	return
NKRemote:
	Gui, Submit,NoHide
	ifWinExist,NKRemote
	{
		MsgBox,Please exit NKRemote before installing/uninstalling this script
		RegRead, enabled, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdEnable
		if enabled
		{
			GuiControl,,NKRemoteState,1
		}
		else
		{
			GuiControl,,NKRemoteState,0
		}
		return
	}
	if NKRemoteState
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdEnable, 1
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %NKRKey%, PhotoboothSuppressXml, 0
		RegWrite, REG_SZ, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdXML, %newCmd%
		MsgBox,8192,%A_ScriptName%,Script enabled for NKRemote,1000
	}
	else
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdEnable, 0
		MsgBox,8192,%A_ScriptName%,Script disabled for NKRemote,1000
	}
	return
Webcam:
	Gui, Submit,NoHide
	ifWinExist,Webcam Photobooth
	{
		MsgBox,Please exit Webcam Photobooth before installing/uninstalling this script
		RegRead, enabled, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdEnable
		if enabled
		{
			GuiControl,,WebcamState,1
		}
		else
		{
			GuiControl,,WebcamState,0
		}
		return
	}
	if WebcamState
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdEnable, 1
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %WebKey%, PhotoboothSuppressXml, 0
		RegWrite, REG_SZ, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdXML, %newCmd%
		MsgBox,8192,%A_ScriptName%,Script enabled for Webcam Photobooth,1000
	}
	else
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdEnable, 0
		MsgBox,8192,%A_ScriptName%,Script disabled for Webcam Photobooth,1000
	}
	return
PSRemote:
	Gui, Submit,NoHide
	ifWinExist,PSRemote -
	{
		MsgBox,Please exit PSRemote before installing/uninstalling this script
		RegRead, enabled, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdEnable
		if enabled
		{
			GuiControl,,PSRemoteState,1
		}
		else
		{
			GuiControl,,PSRemoteState,0
		}
		return
	}
	if PSRemoteState
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdEnable, 1
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %PSRKey%, PhotoboothSuppressXml, 0
		RegWrite, REG_SZ, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdXML, %newCmd%
		MsgBox,8192,%A_ScriptName%,Script enabled for PSRemote,1000
	}
	else
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdEnable, 0
		MsgBox,8192,%A_ScriptName%,Script disabled for PSRemote,1000
	}
	return

GuiClose:
ButtonClose:
	exitapp
}
